/**
 * Class ComplexRoot - Implements the Newton-Raphson method of root finding
 * for Polynomials.
 *
 * @author Matt Johnston
 */

public class ComplexRoot {

private Polynomial poly;

// this is used to count number of iterations
private int itercount;

private double maxprecision; //this is for finding the roots

// test is a test complex value which is used for efficiency in findRoot
Complex test = new Complex();

// these values are used in evalDivDeriv for efficiency, they are created once
Complex prevcomplex = new Complex();
Complex nextcomplex = new Complex();
Complex derivative = new Complex();

private static final int MAXROOTITERATION = 200;

/**
 * Creates a new ComplexRoot object. This object implements the Newton-Raphson
 * method of finding roots for Polynomials.
 * @param pol The Polynomial to use.
 * @param precision The precision to which we should evaluate the root.
 * This value determines how close the value of the Polynomial should be until
 * a value is returned. Smaller values will give greater accuracy but increase
 * the time taken.
 */
public ComplexRoot(Polynomial pol, double precision) {

	setPolynomial(pol);
	setPrecision(precision);

}

/**
 * Sets the Polynomial to use with this object.
 * @param pol This is the Polynomial to use.
 */
public void setPolynomial(Polynomial pol) {

	poly = pol;

}
/**
 * Sets the precision to use when finding roots.
 * @param precision The precision to which we should evaluate the root.
 * This value determines how close the value of the Polynomial should be until
 * a value is returned. Smaller values will give greater accuracy but increase
 * the time taken.
 */
public void setPrecision(double precision) {

	maxprecision = precision;

}

/**
 * Implements the Newton Raphson method to find a root of the polynomial
 * @param initial The starting point to begin iteration
 * @return Returns a complex number which is a root of the polynomial,
 * or null if it does not converge after 200 iterations.
*/
public Complex findRoot(Complex initial) {

	test.setComplex(initial);
	Complex tempval = test;
	itercount = 0;

	while (tempval.modulusSquare() > maxprecision) { //if not close enough to 0

		tempval = evalDivDeriv(test);
		// return null if zero derivative
		if (tempval== null) return null;

		test.subtract(tempval);

		if (itercount >= MAXROOTITERATION) return null; //if it doesn't converge
		itercount++;

	}
	return test;

}

/**
 * This method implements "Horner's Algorithm" for use in the Newton-Raphson
 * Method of determining roots of a polynomial.
 * @param x The complex value to evaluate the polynomial at.
 * @return The value of the polynomial at x, divided by the value of the
 * derivative of the polynomial at x. If the value cannot be determined
 * (the derivative is zero for example) then null is returned.
 */
private Complex evalDivDeriv(Complex x) {

	final int thisdegree = poly.getActualDegree();
	final int thisdegminus = thisdegree - 1;

	// powerlist is a list of {x^0, x, x^2, ... , x^thisdegminus}
	Complex[] powerlist = x.powerList(thisdegminus);

	nextcomplex.setZero();
	nextcomplex.setRe(poly.getCoefficient(thisdegree));

	derivative.setZero();

	// this loop is the core of the Horner's Algorithm
	// comments are provided as if the complex variables were primitives:
	for (int i = 0; i < thisdegree; i++) {

		//prevcomplex = nextcomplex
		prevcomplex.setComplex(nextcomplex);

		//nextcomplex = prevcomplex * x
		prevcomplex.multiply(x,nextcomplex);

		//nextcomplex += poly.getCoefficient(thisdegminus-i)
		nextcomplex.addReal(poly.getCoefficient(thisdegminus-i));

		//prevcomplex = prevcomplex * (powerlist[thisdegminus-i])
		prevcomplex.multiply(powerlist[thisdegminus - i], prevcomplex);

		//derivative += prevcomplex
		derivative.add(prevcomplex);
	}

	//prevent divide by zero error
	if (derivative.modulusSquare()==0) return null;

	//prevcomplex = nextcomplex / derivative
	nextcomplex.divide(derivative, prevcomplex);
	return prevcomplex;

}

/**
 * Determines the number of iterations required to find a root for the
 * Polynomial.
 * @param initial The initial value to start from, when using the Newton-Raphson
 * method.
 * @return The number of iterations required for the Polynomial to converge
 * on a root. If the polynomial does not converge (within 200 iterations),
 * the maximum iteration count of 200 is returned.
 */
public int findRootIterCount(Complex initial) {

	findRoot(initial);

	return	itercount;

}

}

