import java.awt.*;

/**
 * Class PolynomialPlotter - Draws a polynomial graph
 * 
 * @author Matt Johnston 
 */

public class PolynomialPlotter
{

private Canvas drawcanvas;
private Polynomial poly;
private int canvaswidth;
private int canvasheight; 

private int leftmargin = 55;
private int rightmargin = 20;
private int topmargin = 10;
private int bottommargin = 20;
private int lefttextmargin = 7;

private int plotwidth, plotheight;

private double left = -10.0;
private double right = 10.0;
private double bottom = -10.0;
private double top = 10.0;
private boolean autorange = true;

private double xgrid = 1;
private double ygrid = 1;
private int xgridplot = 10; //number of "ticks" or gridlines in x
private int ygridplot = 10;
private boolean drawgrid = true;

private final int MINCANVASSIZE = 100;
private final static int INFINITY = 2147473647; // largest integer possible

private Color bordercolor = java.awt.Color.black;
private Color axescolor = java.awt.Color.black;
private Color gridcolor = new java.awt.Color(200,200,200);
private Color curvecolor = java.awt.Color.red;
private Color textcolor = java.awt.Color.black;

private static final int SCALETICKSIZE = 3;
private static final int XAXISLEFT = -8;
private static final int XAXISUP = 14;
private static final int YAXISUP = 7;

/**
 * Creates a new PolynomialPlotter with default values. 
 * The default width and height are both 500.
 * The Polynomial must be set before plotting.
 */
public PolynomialPlotter() {

	canvaswidth = 500;
	canvasheight = 500;

}

/**
 * Creates a new PolynomialPlotter with default values, for a specified
 * Polynomial.
 * @param polynom The polynomial to use for this PolynomialPlotter
 */
public PolynomialPlotter(Polynomial poly) {

	this();
	setPolynomial(poly);

}

/**
 * Set the polynomial to draw.
 * @param polynom The polynomial to use for drawing.
 */
public void setPolynomial(Polynomial poly) {

	this.poly = poly;

}

/**
 * Set the canvas size.
 * @param width The canvas width to use. This must be at least 100.
 * @param height The canvas height to use. This must be at least 100.
 */
public void setCanvasSize(int width, int height) {

	if ((width < MINCANVASSIZE) || (height < MINCANVASSIZE)) {
		System.out.println("Warning: width and height must be at least "
							+ MINCANVASSIZE + ", they have now both been"
							+ " set to " + MINCANVASSIZE);

		canvaswidth = MINCANVASSIZE;
		canvasheight = MINCANVASSIZE;
		return;
	}

	canvaswidth = width;
	canvasheight = height;

}

/** 
 * Set the horizontal range of the graph.
 * The left value should be less than the right value.
 * @param left The leftmost extent of the graph. The default is -10.
 * @param right The rightmost extent of the graph. The default is 10.
 */
public void setHorizRange(double left, double right) {

	if (left > right) {
		System.out.println("Warning: left is greater than right,"
								+ " they have now been swapped");
		this.left = right;
		this.right = right;
		return;
	}


	this.left = left;
	this.right = right;

}

/**
 * Sets the vertical range of the graph, or sets auto-scaling.
 * The bottom value should be less than the top value.
 * @param auto If this is true, autoscaling is used. When auto is set,
 * values for bottom and top are ignored.
 * @param bottom This is the bottom extent of the graph. The default is -10.
 * @param top This is the top extent of the graph. The default is 10.
 */
public void setVertRange(boolean auto, double bottom, double top) {

	autorange = auto;
	if (auto) return;

	if (bottom > top) {
		System.out.println("Warning: bottom is greater than top,"
								+" they have now been swapped");
		this.bottom = top;
		this.top = bottom;
		return;
	}

	this.bottom = bottom;
	this.top = top;

}

/**
 * Set whether to draw a grid on the graph.
 * @param drawgrid If this is true, a grid will be drawn on the graph.
 * The default is true.
 */
public void setDrawGrid(boolean drawgrid) {

	this.drawgrid = drawgrid;

}

/**
 * Plot the polynomial graph. The polynomial must have been specified,
 * all other values can be left as their default values.
 */
public void plot() {

	if (poly == null) {
		System.out.println("Polynomial not specified");
		System.exit(1);
	}

	plotwidth = canvaswidth - (leftmargin + rightmargin);
	plotheight = canvasheight - (topmargin + bottommargin);

	// determine the points of the graph
	double[] pointlist = calcPoints();

	// set up scaling
	findRange(pointlist);

	drawcanvas = new Canvas("Polynomial Graph", canvaswidth, canvasheight);
	drawcanvas.setVisible(true);

	if (drawgrid)
		drawGrid(); //draw the grid and scale
	else
		drawScale(); //draw just the scale

	drawAxes();

	drawCurve(pointlist);

	drawBorderBox();
}

private double[] calcPoints() {

	double interval = (right - left) / plotwidth;

	double[] points = new double[plotwidth];

	double pos = left;
	for (int index = 0; index < plotwidth; index++) {
		points[index] = poly.evaluate(pos);
		pos += interval;
	}

	return points;

}

private void drawCurve(double[] points) {

	int previousx = 0;
	double previousy = points[0];

	drawcanvas.setForegroundColour(curvecolor);

	for (int i = 1; i < points.length; i++) {
		drawcanvas.drawLine(previousx + leftmargin,
							pointToCanvasY(previousy),
							i + leftmargin,
							pointToCanvasY(points[i]) );
		previousx = i;
		previousy = points[i];
	}

}

// The following 3 methods handle all scaling factors, including margins etc.
private int pointToCanvasX(double x) {

	double xpos = x;

	if (x < left) xpos = left;
	if (x > right) xpos = right;
	return (int)(leftmargin + ((xpos - left) * (plotwidth / (right - left))));

}

private int pointToCanvasY(double y) {

	double ypos = y;

	if (y < bottom) ypos = bottom;
	if (y > top) ypos = top;
	return (int)(topmargin + ((ypos - top) * (plotheight / (bottom - top))));

}

private void findRange(double[] points) {

	if (autorange) {

//		double baserange = (right - left) * 10;

		double max = -INFINITY;
		double min = INFINITY;

		for (int i = 0; i < points.length; i++) {
			if (points[i] < min) min = points[i];
			if (points[i] > max) max = points[i];
		}

		if (max == min) {
			max = 1.5 * max;
			min = 0.5 * min;
		}

		bottom = min;
		top = max;
	}

	ygrid = (top - bottom) / xgridplot;
	xgrid = (right - left) / ygridplot;

}

// used to draw text at a particular point, handles colour changing
private void drawText(double value, int x, int y) {

	java.text.DecimalFormat df;

	// numbers between 0.001 and 1000 are shown normally, others are 2.3E5 form
	if ((Math.abs(value) < 1000) && (Math.abs(value) > 0.001) || (value == 0))
		df = new java.text.DecimalFormat("0.##");
	else 
		df = new java.text.DecimalFormat("#0.#E0");

	df.setDecimalSeparatorAlwaysShown(false);

	Color oldcolor = drawcanvas.getForegroundColour();
	drawcanvas.setForegroundColour(textcolor);
	drawcanvas.drawString(df.format(value), x, y);
	drawcanvas.setForegroundColour(oldcolor);

}

private void drawBorderBox() {

	drawcanvas.setForegroundColour(bordercolor);
	drawcanvas.draw(new Rectangle(leftmargin,topmargin,plotwidth,plotheight));

}

private void drawScale() {

	//x axis
	for (double pos = (int)(left / xgrid) * xgrid; pos <= right; pos += xgrid) {
	//pos is set to leftmost position within range

		drawcanvas.drawLine(pointToCanvasX(pos),
							plotheight + topmargin - SCALETICKSIZE,
							pointToCanvasX(pos),
							plotheight + topmargin);

		drawText(pos,
				pointToCanvasX(pos) + XAXISLEFT,
				plotheight + topmargin + XAXISUP);

	}

	//y axis
	for (double pos = (int)(bottom / ygrid) * ygrid; pos <= top; pos+=ygrid) {
	//pos is set to the lowermost position within range

		drawcanvas.drawLine(leftmargin,
							pointToCanvasY(pos),
							leftmargin + SCALETICKSIZE,
							pointToCanvasY(pos));

		drawText(pos, lefttextmargin, pointToCanvasY(pos) + YAXISUP);

	}

}

private void drawGrid() {

	drawcanvas.setForegroundColour(gridcolor);

    //x axis
	for (double pos = (int)(left / xgrid) * xgrid; pos <= right; pos += xgrid) {

		drawcanvas.drawLine(pointToCanvasX(pos),
							topmargin,
							pointToCanvasX(pos),
							plotheight + topmargin);

		drawText(pos,
				pointToCanvasX(pos) + XAXISLEFT,
				plotheight + topmargin + XAXISUP);

	}

	//y axis
	for (double pos = (int)(bottom / ygrid) * ygrid; pos <= top; pos+=ygrid) {

		drawcanvas.drawLine(leftmargin,
							pointToCanvasY(pos),
							leftmargin + plotwidth,
							pointToCanvasY(pos));

		drawText(pos, lefttextmargin, pointToCanvasY(pos) + YAXISUP);

	}

}

private void drawAxes() {

	drawcanvas.setForegroundColour(axescolor);

	drawcanvas.drawLine(pointToCanvasX(0),
						topmargin,
						pointToCanvasX(0),
						topmargin + plotheight);

	drawcanvas.drawLine(leftmargin,
						pointToCanvasY(0),
						leftmargin + plotwidth,
						pointToCanvasY(0));

}

}
